;=========================================================================
; Minimal window capture demo, modal dialog as main.
;
; With the mouse cursor in the client area, press and hold the left
; mouse button, drag the cursor to the target window, then release
; the button.
;=========================================================================

include \GeneSys\include\GeneSysRt.inc
include \GeneSys\macros\macros.asm
include xmacros.asm
include imdialog.asm
include support.asm
include crosshair_cursor.asm

;=========================================================================
.data
    hCursor HCURSOR 0
.code
;=========================================================================

DialogProc proc hwndDlg:HWND, uMsg:UINT, wParam:WPARAM, lParam:LPARAM

    LOCAL hdcScreen:HDC, hdcWnd:HDC, hdcBM:HDC
    LOCAL hwndTarget:HWND
    LOCAL wd:DWORD, ht:DWORD
    LOCAL hbmp:HBITMAP
    LOCAL rc:RECT
    LOCAL pt:POINT

    SWITCH uMsg

      CASE WM_INITDIALOG

        ;------------------------------------------------
        ; If the system cursor size is 32x32, create our
        ; crosshair custom cursor and save the handle.
        ;------------------------------------------------

        invoke GetSystemMetrics, SM_CXCURSOR
        push eax
        invoke GetSystemMetrics, SM_CYCURSOR
        pop edx
        .IF edx == 32 && eax == 32
          invoke GetModuleHandle, NULL
          invoke CreateCursor, eax, 15, 15, 32, 32,
                               ADDR cursor_and_mask,
                               ADDR cursor_xor_mask
          mov hCursor, eax
        .ENDIF

      CASE WM_LBUTTONDOWN

        ;---------------------------------------------------------
        ; If we have a custom cursor, make it the current cursor.
        ;---------------------------------------------------------

        .IF hCursor
          invoke SetCursor, hCursor
        .ENDIF

        ;--------------------
        ; Capture the mouse.
        ;--------------------

        invoke SetCapture, hwndDlg

      CASE WM_LBUTTONUP

        ;-----------------------------------------------------------
        ; Get the client coordinates of the cursor and save them in
        ; our POINT structure. The 16-bit coordinate values must be
        ; sign extended to 32 bits because client coordinates to the
        ; left of and/or above the client area are negative.
        ;-----------------------------------------------------------

        mov eax, loword(lParam)
        movsx eax, ax
        mov pt.x, eax
        mov eax, hiword(lParam)
        movsx eax, ax
        mov pt.y, eax

        ;-------------------------------------------------------
        ; Convert the client coordinates to screen coordinates.
        ;-------------------------------------------------------

        invoke ClientToScreen, hwndDlg, ADDR pt

        ;--------------------------------------------------
        ; Get the handle of the window beneath the cursor.
        ;--------------------------------------------------

        invoke WindowFromPoint, pt.x, pt.y
        mov hwndTarget, eax

        ;-------------------------------------------
        ; Remove our custom cursor from the screen.
        ;-------------------------------------------

        invoke SetCursor, NULL

        ;----------------------------
        ; Release the mouse capture.
        ;----------------------------

        invoke ReleaseCapture

        ;----------------------------------------------------
        ; Bring the target window to the top of the Z-order.
        ;----------------------------------------------------

        invoke SetForegroundWindow, hwndTarget

        ;--------------------------------------
        ; Get a DC for the screen and create a
        ; compatible memory DC for the bitmap.
        ;--------------------------------------

        invoke GetDC, NULL
        mov hdcScreen, eax
        invoke CreateCompatibleDC, hdcScreen
        mov hdcBM, eax

        ;---------------------------------------------
        ; Get the dimensions of the target window and
        ; calculate the bitmap width and height.
        ;---------------------------------------------

        invoke GetWindowRect, hwndTarget, ADDR rc

        mov eax, rc.right
        sub eax, rc.left
        mov wd, eax
        mov eax, rc.bottom
        sub eax, rc.top
        mov ht, eax

        ;-------------------------------------------------
        ; Create a compatible bitmap to store the window
        ; and select it into the memory DC.
        ;-------------------------------------------------

        invoke CreateCompatibleBitmap, hdcScreen, wd, ht
        mov hbmp, eax
        invoke SelectObject, hdcBM, hbmp

        ;---------------------------------
        ; Get a DC for the target window.
        ;---------------------------------

        invoke GetWindowDC, hwndTarget
        mov hdcWnd, eax

        ;--------------------------------
        ; Copy the window to the bitmap.
        ;--------------------------------

        invoke BitBlt, hdcBM, 0, 0, wd, ht, hdcWnd, 0, 0, SRCCOPY

        ;----------------------------
        ; Copy the bitmap to a file.
        ;----------------------------

        invoke CreateBitmapFile, hbmp, 24, chr$("cap.bmp")

        ;-----------
        ; Clean up.
        ;-----------

        invoke DeleteObject, hbmp
        invoke DeleteDC, hdcBM
        invoke ReleaseDC, NULL, hdcScreen
        invoke ReleaseDC, NULL, hdcWnd

      CASE WM_COMMAND

        SWITCH loword(wParam)

          CASE IDCANCEL

            invoke EndDialog, hwndDlg, NULL

        ENDSW

      CASE WM_CLOSE

        invoke EndDialog, hwndDlg, NULL

    ENDSW

    xor eax, eax
    ret

DialogProc endp

;=========================================================================
start:
;=========================================================================

    invoke DefineDialog, chr$("Window Capture Demo"),
                         WS_OVERLAPPED or WS_SYSMENU \
                         or DS_CENTER,
                         0, 0, 100, 75,
                         0, 0,
                         0

    invoke CreateModalDialog, 0, ADDR DialogProc, 0

    invoke ExitProcess, 0

;=========================================================================

end start
