;=========================================================================
; Dot-matrix digital clock demo, modal dialog as main.
;
; This source is dependent on the resource definition clock.rc,
; the resource compiler, and the linker to include the bitmap
; resources in the EXE. The alternative, loading the individual
; bitmap files at runtime, would produce a much smaller EXE but
; it would work correctly only if the bitmap files were present
; at runtime.
;=========================================================================

include \GeneSys\include\GeneSysRt.inc
include \GeneSys\macros\macros.asm
include xmacros.asm
include imdialog.asm
include support.asm

IDC_H1    equ 101
IDC_H2    equ 102
IDC_C1    equ 103
IDC_M1    equ 104
IDC_M2    equ 105
IDC_C2    equ 106
IDC_S1    equ 107
IDC_S2    equ 108

IDB_0     equ 1000
IDB_1     equ 1001
IDB_2     equ 1002
IDB_3     equ 1003
IDB_4     equ 1004
IDB_5     equ 1005
IDB_6     equ 1006
IDB_7     equ 1007
IDB_8     equ 1008
IDB_9     equ 1009
IDB_C     equ 1010

;=========================================================================
.data

    dtuX        dd 0
    dtuY        dd 0

    hwndH1      HWND    0
    hwndH2      HWND    0
    hwndC1      HWND    0
    hwndM1      HWND    0
    hwndM2      HWND    0
    hwndC2      HWND    0
    hwndS1      HWND    0
    hwndS2      HWND    0

    hbmpDigits  HBITMAP 10 dup(0)
    hbmpColon   HBITMAP 0

    systime     SYSTEMTIME <>
.code
;=========================================================================

UpdateTime proc

    invoke GetLocalTime, ADDR systime

    ;---------------------------------------------------------------
    ; For the following code, dividing the hour/minute/second by 10
    ; leaves the value of the most significant digit in EAX and the
    ; value of the least significant digit in EDX. The digit values
    ; are then used as an index into the hbmpDigits array, and the
    ; indexed bitmap is associated with the proper control.
    ;---------------------------------------------------------------

    movzx eax, systime.wHour
    mov ecx, 10
    xor edx, edx
    div ecx
    mov eax, [hbmpDigits+eax*4]
    push edx
    SETIMAGE hwndH1, eax
    pop edx
    mov eax, [hbmpDigits+edx*4]
    SETIMAGE hwndH2, eax

    movzx eax, systime.wMinute
    mov ecx, 10
    xor edx, edx
    div ecx
    mov eax, [hbmpDigits+eax*4]
    push edx
    SETIMAGE hwndM1, eax
    pop edx
    mov eax, [hbmpDigits+edx*4]
    SETIMAGE hwndM2, eax

    movzx eax, systime.wSecond
    mov ecx, 10
    xor edx, edx
    div ecx
    mov eax, [hbmpDigits+eax*4]
    push edx
    SETIMAGE hwndS1, eax
    pop edx
    mov eax, [hbmpDigits+edx*4]
    SETIMAGE hwndS2, eax

    ret

UpdateTime endp

;=========================================================================

DeleteBitmapResources proc

    invoke DeleteObject, hbmpDigits+4*0
    invoke DeleteObject, hbmpDigits+4*1
    invoke DeleteObject, hbmpDigits+4*2
    invoke DeleteObject, hbmpDigits+4*3
    invoke DeleteObject, hbmpDigits+4*4
    invoke DeleteObject, hbmpDigits+4*5
    invoke DeleteObject, hbmpDigits+4*6
    invoke DeleteObject, hbmpDigits+4*7
    invoke DeleteObject, hbmpDigits+4*8
    invoke DeleteObject, hbmpDigits+4*9
    invoke DeleteObject, hbmpColon

    ret

DeleteBitmapResources endp

;=========================================================================

DialogProc proc hwndDlg:HWND, uMsg:UINT, wParam:WPARAM, lParam:LPARAM

    LOCAL rcDlg:RECT
    LOCAL ps:PAINTSTRUCT

    SWITCH uMsg

      CASE WM_INITDIALOG

        ;-----------------------------------------------------------
        ; Load the digit bitmaps from our EXE and save the handles.
        ;-----------------------------------------------------------

        mov hbmpDigits+0*4, loadimage(IDB_0)
        mov hbmpDigits+1*4, loadimage(IDB_1)
        mov hbmpDigits+2*4, loadimage(IDB_2)
        mov hbmpDigits+3*4, loadimage(IDB_3)
        mov hbmpDigits+4*4, loadimage(IDB_4)
        mov hbmpDigits+5*4, loadimage(IDB_5)
        mov hbmpDigits+6*4, loadimage(IDB_6)
        mov hbmpDigits+7*4, loadimage(IDB_7)
        mov hbmpDigits+8*4, loadimage(IDB_8)
        mov hbmpDigits+9*4, loadimage(IDB_9)
        mov hbmpColon, loadimage(IDB_C)

        ;-------------------------------------------------------------
        ; Get and save the handles for the SS_BITMAP static controls.
        ;-------------------------------------------------------------

        invoke GetDlgItem, hwndDlg, IDC_H1
        mov hwndH1, eax
        invoke GetDlgItem, hwndDlg, IDC_H2
        mov hwndH2, eax
        invoke GetDlgItem, hwndDlg, IDC_C1
        mov hwndC1, eax
        invoke GetDlgItem, hwndDlg, IDC_M1
        mov hwndM1, eax
        invoke GetDlgItem, hwndDlg, IDC_M2
        mov hwndM2, eax
        invoke GetDlgItem, hwndDlg, IDC_C2
        mov hwndC2, eax
        invoke GetDlgItem, hwndDlg, IDC_S1
        mov hwndS1, eax
        invoke GetDlgItem, hwndDlg, IDC_S2
        mov hwndS2, eax

        ;---------------------------------------------
        ; The colons are fixed so there is no need to
        ; draw them in the UpdateTimer procedure.
        ;---------------------------------------------

        SETIMAGE hwndC1, hbmpColon
        SETIMAGE hwndC2, hbmpColon

        ;----------------------------------------------------
        ; Create a timer the will fire once every second and
        ; post a WM_TIMER message to our message queue.
        ;----------------------------------------------------

        invoke SetTimer, hwndDlg, 1, 1000, NULL

        ;---------------------------------------
        ; Perform the first update now, instead
        ; of waiting for the timer to fire.
        ;---------------------------------------

        invoke UpdateTime

      CASE WM_CTLCOLORDLG

        ;------------------------------------------
        ; Set the client area background to black.
        ;------------------------------------------

        invoke GetStockObject, BLACK_BRUSH
        ret

      CASE WM_TIMER

        invoke UpdateTime

      CASE WM_COMMAND

        SWITCH loword(wParam)

          CASE IDCANCEL

            ;--------------------
            ; Clean up and exit.
            ;--------------------

            invoke KillTimer, hwndDlg, 1
            invoke DeleteBitmapResources
            invoke EndDialog, hwndDlg, NULL

        ENDSW

      CASE WM_CLOSE

        ;--------------------
        ; Clean up and exit.
        ;--------------------

        invoke KillTimer, hwndDlg, 1
        invoke DeleteBitmapResources
        invoke EndDialog, hwndDlg, NULL

    ENDSW

    xor eax, eax
    ret

DialogProc endp

;=========================================================================
start:
;=========================================================================

    ;---------------------------------------------------------------
    ; Get a width and height in DTUs that will size the client area
    ; close to the specified pixel width and height, regardless of
    ; the system font settings.
    ;---------------------------------------------------------------

    invoke ClientPixelWidthToDialogWidth, 624
    mov dtuX, eax
    invoke ClientPixelHeightToDialogHeight, 160
    mov dtuY, eax

    invoke DefineDialog, chr$("Digital Clock Demo"),
                         WS_OVERLAPPED or WS_SYSMENU \
                         or DS_CENTER,
                         0, 0, dtuX, dtuY,
                         0, 0,
                         8

    ;----------------------------------------------------------
    ; Specify the bitmap control positions in pixels converted
    ; to DTUs, so the position will not vary with the system
    ; font settings. The digit bitmaps are 84 pixels wide and
    ; the colon bitmap 40 pixels wide.
    ;----------------------------------------------------------

    invoke DefineBitmap, 0, pdx(20), pdy(28), 0, IDC_H1
    invoke DefineBitmap, 0, pdx(104), pdy(28), 0, IDC_H2
    invoke DefineBitmap, 0, pdx(188), pdy(28), 0, IDC_C1
    invoke DefineBitmap, 0, pdx(228), pdy(28), 0, IDC_M1
    invoke DefineBitmap, 0, pdx(312), pdy(28), 0, IDC_M2
    invoke DefineBitmap, 0, pdx(396), pdy(28), 0, IDC_C2
    invoke DefineBitmap, 0, pdx(436), pdy(28), 0, IDC_S1
    invoke DefineBitmap, 0, pdx(520), pdy(28), 0, IDC_S2

    invoke CreateModalDialog, 0, ADDR DialogProc, 0

    invoke ExitProcess, 0

;=========================================================================

end start




