;=========================================================================
; Create bitmap file demo, modal dialog as main.
;=========================================================================

include \GeneSys\include\GeneSysRt.inc
include \GeneSys\macros\macros.asm
include xmacros.asm
include imdialog.asm
include support.asm

;=========================================================================
.data
    dtuX  dd      0
    dtuY  dd      0
    hbmp  HBITMAP 0
.code
;=========================================================================

;------------------------------------------------------------------
; This procedure creates our Device Dependent Bitmap and returns
; the handle. Ths bitmap will consist of three overlapping, filled
; circles, one for each of the primary colors. The centers of the
; circles will form an equilateral triangle with the length of the
; sides equal to the radius of the circles (90 in this case), with
; the coordinates of the centers pre-calculated as follows (angles
; specified in degrees):
; Red:    x=149, y=149-r*cos(30)/2
; Green:  x=149-r/2, y=149+r*cos(30)/2
; Blue:   x=149+r/2, y=149+r*cos(30)/2
;------------------------------------------------------------------

CreateDDB proc

    LOCAL hdcScreen:HDC, hdcRed:HDC, hdcGreen:HDC, hdcBlue:HDC
    LOCAL hbmpRed:HBITMAP, hbmpGreen:HBITMAP, hbmpBlue:HBITMAP
    LOCAL hbrush:HBRUSH
    LOCAL rc:RECT

    ;--------------------------------------------------------------
    ; Get a DC for the screen and create the compatible memory DCs
    ; and the compatible bitmaps, one for each color.
    ;--------------------------------------------------------------

    invoke GetDC, NULL
    mov hdcScreen, eax

    invoke CreateCompatibleDC, hdcScreen
    mov hdcRed, eax
    invoke CreateCompatibleDC, hdcScreen
    mov hdcGreen, eax
    invoke CreateCompatibleDC, hdcScreen
    mov hdcBlue, eax

    invoke CreateCompatibleBitmap, hdcScreen, 300, 300
    mov hbmpRed, eax
    invoke CreateCompatibleBitmap, hdcScreen, 300, 300
    mov hbmpGreen, eax
    invoke CreateCompatibleBitmap, hdcScreen, 300, 300
    mov hbmpBlue, eax

    ;-------------------------------------------------------
    ; Select our bitmaps into the corresponding memory DCs.
    ;-------------------------------------------------------

    invoke SelectObject, hdcRed, hbmpRed
    invoke SelectObject, hdcGreen, hbmpGreen
    invoke SelectObject, hdcBlue, hbmpBlue

    ;---------------------------------------------------------------
    ; Fill each of the bitmaps with black. Note that the right and
    ; bottom coordinates have been extended by one pixel each so the
    ; bottom row and rightmost column will contain black instead of
    ; random garbage. See the Microsoft documentation for the
    ; FillRect function for more information.
    ;---------------------------------------------------------------

    invoke SetRect, ADDR rc, 0, 0, 300, 300
    invoke GetStockObject, BLACK_BRUSH
    mov hbrush, eax
    invoke FillRect, hdcRed, ADDR rc, hbrush
    invoke FillRect, hdcGreen, ADDR rc, hbrush
    invoke FillRect, hdcBlue, ADDR rc, hbrush

    ;----------------------------------------------
    ; Draw the filled circles, each in its own DC.
    ;----------------------------------------------

    invoke DrawFilledCircle, hdcRed, 149, 110, 90, rgb(255,0,0)
    invoke DrawFilledCircle, hdcGreen, 104, 188, 90, rgb(0,255,0)
    invoke DrawFilledCircle, hdcBlue, 194, 188, 90, rgb(0,0,255)

    ;---------------------------------------------------------
    ; Combine the other bitmaps into the red bitmap using the
    ; Boolean OR operator (specified by SRCPAINT). This step
    ; is essential to mix the colors for the desired effect.
    ;---------------------------------------------------------

    invoke BitBlt, hdcRed, 0, 0, 300, 300, hdcGreen, 0, 0, SRCPAINT
    invoke BitBlt, hdcRed, 0, 0, 300, 300, hdcBlue, 0, 0, SRCPAINT

    ;------------------------------------------------------------
    ; Delete/release the DCs and bitmaps that we no longer need.
    ;------------------------------------------------------------

    invoke DeleteDC, hdcRed
    invoke DeleteDC, hdcGreen
    invoke DeleteDC, hdcBlue
    invoke DeleteObject, hbmpGreen
    invoke DeleteObject, hbmpBlue
    invoke ReleaseDC, NULL, hdcScreen

    ;--------------------------------------------
    ; Return the handle for the combined bitmap.
    ;--------------------------------------------

    mov eax, hbmpRed
    ret

CreateDDB endp

;=========================================================================

DialogProc proc hwndDlg:HWND, uMsg:UINT, wParam:WPARAM, lParam:LPARAM

    LOCAL hdcBM:HDC
    LOCAL ps:PAINTSTRUCT

    LOCAL rc:RECT

    SWITCH uMsg

      CASE WM_INITDIALOG

        ;------------------------------------------------------
        ; Adjust the client area to 300x300 pixels, precisely.
        ;------------------------------------------------------

        invoke SetClientSize, hwndDlg, 300, 300

        ;------------------------------------------------------
        ; Create the the bitmap, save the handle, and save the
        ; bitmap to a file. The CreateBitmapFile source is in
        ; support.asm.
        ;------------------------------------------------------

        invoke CreateDDB
        mov hbmp, eax
        invoke CreateBitmapFile, hbmp, 24, chr$("sample.bmp")

      CASE WM_PAINT

        invoke BeginPaint, hwndDlg, ADDR ps

        ;--------------------------------------------
        ; Create a memory DC that is compatible with
        ; the paint DC and save the handle.
        ;--------------------------------------------

        invoke CreateCompatibleDC, ps.hdc
        mov hdcBM, eax

        ;------------------------------------------------------------
        ; Select our bitmap into the memory DC. We have no need here
        ; to save the previously selected object, returned by the
        ; SelectObject function, because we will delete the DC when
        ; we are finished with it.
        ;
        ; Note that a bitmap cannot be selected into more than one
        ; DC at a time, so if we were not deleting the DC it would
        ; be necessary to replace the new object with the original,
        ; default object, as otherwise the call to the SelectObject
        ; function would fail on the next paint.
        ;------------------------------------------------------------

        invoke SelectObject, hdcBM, hbmp

        ;---------------------------------------------------
        ; Display the bitmap by copying it to the paint DC.
        ;---------------------------------------------------

        invoke BitBlt, ps.hdc, 0, 0, 300, 300, hdcBM, 0, 0, SRCCOPY

        invoke DeleteDC, hdcBM

        invoke EndPaint, hwndDlg, ADDR ps

      CASE WM_COMMAND

        SWITCH loword(wParam)

          CASE IDCANCEL

            ;--------------------
            ; Clean up and exit.
            ;--------------------

            invoke DeleteObject, hbmp
            invoke EndDialog, hwndDlg, NULL

        ENDSW

      CASE WM_CLOSE

        ;--------------------
        ; Clean up and exit.
        ;--------------------

        invoke DeleteObject, hbmp
        invoke EndDialog, hwndDlg, NULL

    ENDSW

    xor eax, eax
    ret

DialogProc endp

;=========================================================================
start:
;=========================================================================

    ;---------------------------------------------------------------
    ; Get a width and height in DTUs that will size the client area
    ; close to the specified pixel width and height, regardless of
    ; the system font settings.
    ;---------------------------------------------------------------

    invoke ClientPixelWidthToDialogWidth, 300
    mov dtuX, eax
    invoke ClientPixelHeightToDialogHeight, 300
    mov dtuY, eax

    invoke DefineDialog, chr$("Create Sample Bitmap"),
                         WS_OVERLAPPED or WS_SYSMENU \
                         or DS_CENTER,
                         0, 0, dtuX, dtuY,
                         0, 0,
                         0

    invoke CreateModalDialog, 0, ADDR DialogProc, 0

    invoke ExitProcess, 0

;=========================================================================

end start
