;========================================================================
; Menu demo, fully programmed (no menu resource) with keyboard
; accelerators, modeless dialog as main.
;
; Keyboard accelerators are easier to implement in a modeless
; dialog because you have direct access to the message loop.
;========================================================================

include \GeneSys\include\GeneSysRt.inc
include \GeneSys\macros\macros.asm
include xmacros.asm
include imdialog.asm
include support.asm

IDM_NEW     equ 201
IDM_OPEN    equ 202
IDM_SAVE    equ 203
IDM_SAVEAS  equ 204
IDM_EXIT    equ 205
IDM_ABOUT   equ 206

;=========================================================================
.data
    hDlg      HWND    0
    hMenu     HMENU   0
    hFileMenu HMENU   0
    hHelpMenu HMENU   0
    hAccel    HACCEL  0
    msg       MSG     <>
.code
;=========================================================================

InitMenu proc

    LOCAL accl[3]   :ACCEL

    invoke CreateMenu
    mov hMenu, eax
    invoke CreateMenu
    mov hFileMenu, eax
    invoke CreateMenu
    mov hHelpMenu, eax

    invoke AppendMenu, hMenu, MF_POPUP, hFileMenu, chr$("&File")
    invoke AppendMenu, hMenu, MF_POPUP, hHelpMenu, chr$("&Help")
    invoke AppendMenu, hFileMenu, MF_STRING, IDM_NEW,
                       chr$("&New",9,"Ctrl+N")
    invoke AppendMenu, hFileMenu, MF_SEPARATOR, 0, 0
    invoke AppendMenu, hFileMenu, MF_STRING, IDM_OPEN,
                       chr$("&Open",9,"Ctrl+O")
    invoke AppendMenu, hFileMenu, MF_SEPARATOR, 0, 0

    invoke AppendMenu, hFileMenu, MF_STRING, IDM_SAVE,
                       chr$("&Save",9,"Ctrl+S")
    invoke AppendMenu, hFileMenu, MF_STRING, IDM_SAVEAS,
                       chr$("Save &As")
    invoke AppendMenu, hFileMenu, MF_SEPARATOR, 0, 0
    invoke AppendMenu, hFileMenu, MF_STRING, IDM_EXIT,
                       chr$("E&xit",9,"Alt+F4")
    invoke AppendMenu, hHelpMenu, MF_STRING, IDM_ABOUT,
                       chr$("&About")

    mov accl[0*SIZEOF ACCEL].fVirt, FCONTROL or FVIRTKEY
    mov accl[0*SIZEOF ACCEL].key, 'N'
    mov accl[0*SIZEOF ACCEL].cmd, IDM_NEW
    mov accl[1*SIZEOF ACCEL].fVirt, FCONTROL or FVIRTKEY
    mov accl[1*SIZEOF ACCEL].key, 'O'
    mov accl[1*SIZEOF ACCEL].cmd, IDM_OPEN
    mov accl[2*SIZEOF ACCEL].fVirt, FCONTROL or FVIRTKEY
    mov accl[2*SIZEOF ACCEL].key, 'S'
    mov accl[2*SIZEOF ACCEL].cmd, IDM_SAVE

    invoke CreateAcceleratorTable, ADDR accl, 3
    mov hAccel, eax

    ret

InitMenu endp

;=========================================================================

DialogProc proc hwndDlg:HWND, uMsg:UINT, wParam:WPARAM, lParam:LPARAM

    LOCAL rcDlg:RECT
    LOCAL ps:PAINTSTRUCT

    SWITCH uMsg

      CASE WM_INITDIALOG

        invoke InitMenu
        invoke SetMenu, hwndDlg, hMenu

      CASE WM_COMMAND

        SWITCH loword(wParam)

          CASE IDM_NEW

            invoke MessageBox, hwndDlg, chr$("New"), chr$(" ") , 0

          CASE IDM_OPEN

            invoke MessageBox, hwndDlg, chr$("Open"), chr$(" ") , 0

          CASE IDM_SAVE

            invoke MessageBox, hwndDlg, chr$("Save"), chr$(" ") , 0

          CASE IDM_SAVEAS

            invoke MessageBox, hwndDlg, chr$("Save As"), chr$(" ") , 0

          CASE IDM_ABOUT

            invoke MessageBox, hwndDlg, chr$("About"), chr$(" ") , 0

          CASE IDM_EXIT, IDCANCEL

            ;--------------------
            ; Clean up and exit.
            ;--------------------

            invoke DestroyMenu, hMenu
            invoke DestroyMenu, hFileMenu
            invoke DestroyMenu, hHelpMenu

            invoke DestroyAcceleratorTable,  hAccel

            invoke DestroyWindow, hwndDlg

        ENDSW

      CASE WM_CLOSE

        ;--------------------
        ; Clean up and exit.
        ;--------------------

        invoke DestroyMenu, hMenu
        invoke DestroyMenu, hFileMenu
        invoke DestroyMenu, hHelpMenu

        invoke DestroyAcceleratorTable,  hAccel

        invoke DestroyWindow, hwndDlg

      CASE WM_DESTROY

        invoke PostQuitMessage, NULL

    ENDSW

    xor eax, eax
    ret

DialogProc endp

;=========================================================================
start:
;=========================================================================

    invoke DefineDialog, chr$("Menu Demo"),
                         WS_OVERLAPPEDWINDOW or \
                         WS_VISIBLE or DS_CENTER,
                         0, 0, 100, 75,
                         0, 0,
                         1

    ;-------------------------------------------------------
    ; The dialog must have at least one control or the menu
    ; will not activate correctly with Alt + access key.
    ; This code uses a button control, but any control will
    ; work. For minimal overhead, use a static text control
    ; with no text.
    ;-------------------------------------------------------

    invoke DefineDefPushButton, chr$("Close"), 0, 35, 40, 30, 10, IDCANCEL

    invoke CreateModelessDialog, 0, ADDR DialogProc, 0
    mov hDlg, eax

    ;----------------------------------------------
    ; The call to TranslateAccelerator adapts this
    ; message loop to process accelerator keys.
    ;----------------------------------------------

  msgLoop:

    invoke GetMessage, ADDR msg, 0, 0, 0
    .IF eax != 0
      invoke TranslateAccelerator, hDlg, hAccel, ADDR msg
      .IF eax == 0
        invoke IsDialogMessage, hDlg, ADDR msg
      .ENDIF
      jmp msgLoop
    .ENDIF

    invoke ExitProcess, eax

;=========================================================================

end start
