;========================================================================
; Menu demo, fully programmed (no menu resource) with keyboard
; accelerators, modal dialog as main.
;
; Because for a modal dialog you do not have direct access to
; the message loop, this code installs a WH_MSGFILTER hook and
; handles the accelerators from the hook procedure, without the
; aid of an accelerator table.
;========================================================================

include \GeneSys\include\GeneSysRt.inc
include \GeneSys\macros\macros.asm
include xmacros.asm
include imdialog.asm
include support.asm

IDM_NEW     equ 201
IDM_OPEN    equ 202
IDM_SAVE    equ 203
IDM_SAVEAS  equ 204
IDM_EXIT    equ 205
IDM_ABOUT   equ 206

;=========================================================================
.data
    hDlg      HWND  0
    hMenu     HMENU 0
    hFileMenu HMENU 0
    hHelpMenu HMENU 0
    hHook     HHOOK 0
    msg       MSG   <>
.code
;=========================================================================

InitMenu proc

    invoke CreateMenu
    mov hMenu, eax
    invoke CreateMenu
    mov hFileMenu, eax
    invoke CreateMenu
    mov hHelpMenu, eax

    invoke AppendMenu, hMenu, MF_POPUP, hFileMenu, chr$("&File")
    invoke AppendMenu, hMenu, MF_POPUP, hHelpMenu, chr$("&Help")
    invoke AppendMenu, hFileMenu, MF_STRING, IDM_NEW,
                       chr$("&New",9,"Ctrl+N")
    invoke AppendMenu, hFileMenu, MF_SEPARATOR, 0, 0
    invoke AppendMenu, hFileMenu, MF_STRING, IDM_OPEN,
                       chr$("&Open",9,"Ctrl+O")
    invoke AppendMenu, hFileMenu, MF_SEPARATOR, 0, 0

    invoke AppendMenu, hFileMenu, MF_STRING, IDM_SAVE,
                       chr$("&Save",9,"Ctrl+S")
    invoke AppendMenu, hFileMenu, MF_STRING, IDM_SAVEAS,
                       chr$("Save &As")
    invoke AppendMenu, hFileMenu, MF_SEPARATOR, 0, 0
    invoke AppendMenu, hFileMenu, MF_STRING, IDM_EXIT,
                       chr$("E&xit",9,"Alt+F4")
    invoke AppendMenu, hHelpMenu, MF_STRING, IDM_ABOUT,
                       chr$("&About")
    ret

InitMenu endp

;=========================================================================

MessageProc proc code:DWORD, wParam:WPARAM, lParam:LPARAM

    push ebx
    .IF code == MSGF_DIALOGBOX
      mov ebx, lParam
      .IF [ebx].MSG.message == WM_KEYDOWN
        invoke GetAsyncKeyState, VK_CONTROL
        .IF eax & 8000h
          SWITCH [ebx].MSG.wParam
            CASE 'N'
              invoke SendMessage, hDlg, WM_COMMAND, IDM_NEW, 0
            CASE 'O'
              invoke SendMessage, hDlg, WM_COMMAND, IDM_OPEN, 0
            CASE 'S'
              invoke SendMessage, hDlg, WM_COMMAND, IDM_SAVE, 0
          ENDSW
        .ENDIF
      .ENDIF
    .ENDIF
    pop ebx

    ;-----------------------------------------
    ; Return whatever CallNextHookEx returns.
    ;-----------------------------------------

    invoke CallNextHookEx, hHook, code, wParam, lParam
    ret

MessageProc endp

;=========================================================================

DialogProc proc hwndDlg:HWND, uMsg:UINT, wParam:WPARAM, lParam:LPARAM

    LOCAL rcDlg:RECT
    LOCAL ps:PAINTSTRUCT

    SWITCH uMsg

      CASE WM_INITDIALOG

        invoke InitMenu
        invoke SetMenu, hwndDlg, hMenu

        mov eax, hwndDlg
        mov hDlg, eax

        invoke GetCurrentThreadId
        invoke SetWindowsHookEx, WH_MSGFILTER, ADDR MessageProc, 0, eax

      CASE WM_COMMAND

        SWITCH loword(wParam)

          CASE IDM_NEW

            invoke MessageBox, hwndDlg, chr$("New"), chr$(" ") , 0

          CASE IDM_OPEN

            invoke MessageBox, hwndDlg, chr$("Open"), chr$(" ") , 0

          CASE IDM_SAVE

            invoke MessageBox, hwndDlg, chr$("Save"), chr$(" ") , 0

          CASE IDM_SAVEAS

            invoke MessageBox, hwndDlg, chr$("Save As"), chr$(" ") , 0

          CASE IDM_ABOUT

            invoke MessageBox, hwndDlg, chr$("About"), chr$(" ") , 0

          CASE IDM_EXIT, IDCANCEL

            ;--------------------
            ; Clean up and exit.
            ;--------------------

            invoke DestroyMenu, hMenu
            invoke DestroyMenu, hFileMenu
            invoke DestroyMenu, hHelpMenu

            invoke UnhookWindowsHookEx, hHook

            invoke EndDialog, hwndDlg, NULL

        ENDSW

      CASE WM_CLOSE

        ;--------------------
        ; Clean up and exit.
        ;--------------------

        invoke DestroyMenu, hMenu
        invoke DestroyMenu, hFileMenu
        invoke DestroyMenu, hHelpMenu

        invoke UnhookWindowsHookEx, hHook

        invoke EndDialog, hwndDlg, NULL

    ENDSW

    xor eax, eax
    ret

DialogProc endp

;=========================================================================
start:
;=========================================================================

    invoke DefineDialog, chr$("Menu Demo"),
                         WS_OVERLAPPEDWINDOW or \
                         WS_VISIBLE or DS_CENTER,
                         0, 0, 100, 75,
                         0, 0,
                         1

    ;-------------------------------------------------------
    ; The dialog must have at least one control or the menu
    ; will not activate correctly with Alt + access key.
    ; This code uses a button control, but any control will
    ; work. For minimal overhead, use a static text control
    ; with no text.
    ;-------------------------------------------------------

    invoke DefineDefPushButton, chr$("Close"), 0, 35, 40, 30, 10, IDCANCEL

    invoke CreateModalDialog, 0, ADDR DialogProc, 0

    invoke ExitProcess, 0

;=========================================================================

end start
