;========================================================================
; Basic framework for a modeless dialog from an in-memory template.
;========================================================================

include \GeneSys\include\GeneSysRt.inc
include \GeneSys\macros\macros.asm
include xmacros.asm
include imdialog.asm
include support.asm

;=========================================================================
.data
    hDlg    HWND 0
    hwndBtn HWND 0
    msg     MSG  <>
.code
;=========================================================================

DialogProc proc hwndDlg:HWND, uMsg:UINT, wParam:WPARAM, lParam:LPARAM

    LOCAL rcDlg:RECT
    LOCAL ps:PAINTSTRUCT

    SWITCH uMsg

      CASE WM_INITDIALOG

        invoke GetDlgItem, hwndDlg, IDCANCEL
        mov hwndBtn, eax

      CASE WM_CTLCOLORDLG

        ;------------------------------------------
        ; Set the client area background to white.
        ;------------------------------------------

        invoke GetStockObject, WHITE_BRUSH
        ret

      CASE WM_SIZE

        invoke GetClientRect, hwndDlg, ADDR rcDlg

      CASE WM_PAINT

        invoke BeginPaint, hwndDlg, ADDR ps
        invoke EndPaint, hwndDlg, ADDR ps

      CASE WM_COMMAND

        SWITCH loword(wParam)

          CASE IDCANCEL

            invoke DestroyWindow, hwndDlg

        ENDSW

      CASE WM_CLOSE

        invoke DestroyWindow, hwndDlg

      CASE WM_DESTROY

        invoke PostQuitMessage, NULL

    ENDSW

    xor eax, eax
    ret

DialogProc endp

;=========================================================================
start:
;=========================================================================

    ;-----------------------------------------------------
    ; For a modeless dialog, for the dialog to be visible
    ; the WS_VISIBLE style must be specified.
    ;-----------------------------------------------------

    invoke DefineDialog, chr$("Title"),
                         WS_OVERLAPPEDWINDOW or \
                         WS_VISIBLE or DS_CENTER,
                         0, 0, 100, 75,
                         0, 0,
                         1

    invoke DefineDefPushButton, chr$("Close"), 0, 35, 50, 30, 10, IDCANCEL

    invoke CreateModelessDialog, 0, ADDR DialogProc, 0
    mov hDlg, eax

    ;--------------------------------------------------------------
    ; The IsDialogMessage function translates and dispatches all
    ; messages for the dialog. TranslateMessage and DispatchMessage
    ; are included for applications where they are required, to
    ; process tool tips, for example, and will be called only if
    ; IsDialogMessage does not process the message.
    ;--------------------------------------------------------------

  msgLoop:

    invoke GetMessage, ADDR msg, 0, 0, 0
    .IF eax != 0
      invoke IsDialogMessage, hDlg, ADDR msg
      .IF eax == 0
        invoke TranslateMessage, ADDR msg
        invoke DispatchMessage, ADDR msg
      .ENDIF
      jmp msgLoop
    .ENDIF

    invoke ExitProcess, eax

;=========================================================================

end start
