;=========================================================================
; Mouse draw demo, modal dialog as main.
;=========================================================================

include \GeneSys\include\GeneSysRt.inc
include \GeneSys\macros\macros.asm
include xmacros.asm
include imdialog.asm
include support.asm
include crosshair_cursor.asm

;=========================================================================
.data
    dtuX      dd      0
    dtuY      dd      0
    fCursor   dd      0
    hdcBM     HDC     0
    hpen      HPEN    0
    hbmp      HBITMAP 0
    hCursor   HCURSOR 0
    tme       TRACKMOUSEEVENT <SIZEOF TRACKMOUSEEVENT, TME_LEAVE>
.code
;=========================================================================

DialogProc proc hwndDlg:HWND, uMsg:UINT, wParam:WPARAM, lParam:LPARAM

    LOCAL hdcClient:HDC
    LOCAL ps:PAINTSTRUCT
    LOCAL pt0:POINT
    LOCAL pt1:POINT
    LOCAL rc:RECT

    SWITCH uMsg

      CASE WM_INITDIALOG

        ;----------------------------------------------------------------
        ; Adjust the client area to the specified pixel size, precisely.
        ;----------------------------------------------------------------

        invoke SetClientSize, hwndDlg, 400, 300

        ;------------------------------------------------
        ; If the system cursor size is 32x32, create our
        ; crosshair custom cursor and save the handle.
        ;------------------------------------------------

        invoke GetSystemMetrics, SM_CXCURSOR
        push eax
        invoke GetSystemMetrics, SM_CYCURSOR
        pop edx
        .IF edx == 32 && eax == 32
          invoke GetModuleHandle, NULL
          invoke CreateCursor, eax, 15, 15, 32, 32,
                               ADDR cursor_and_mask,
                               ADDR cursor_xor_mask
          mov hCursor, eax
        .ENDIF

        ;----------------------------------------------------
        ; Get a DC for the client area of the dialog window,
        ; create a compatible memory DC and a compatible
        ; bitmap to use as a drawing surface, and select the
        ; bitmap into the memory DC.
        ;----------------------------------------------------

        invoke GetDC, hwndDlg
        mov hdcClient, eax

        invoke CreateCompatibleDC, hdcClient
        mov hdcBM, eax
        invoke CreateCompatibleBitmap, hdcClient, 400, 300
        mov hbmp, eax

        ;--------------------------------------------------------
        ; Select the bitmap into the memory DC. We have no need
        ; here to save the previously selected  object, returned
        ; by the SelectObject function, because we will delete
        ; the DC when we are finished with it.
        ;--------------------------------------------------------

        invoke SelectObject, hdcBM, hbmp

        ;--------------------------------------------------
        ; Create a solid black pen with a width of 4, save
        ; the handle, and select it into the bitmap DC.
        ;--------------------------------------------------

        invoke CreatePen, PS_SOLID, 4, 0
        mov hpen, eax
        invoke SelectObject, hdcBM, hpen

        ;-----------------------------------------------------
        ; Fill the bitmap with white. Note that the right and
        ; bottom coordinates have been extended by one pixel
        ; each so the bottom row and rightmost column will
        ; contain white instead of random garbage. See the
        ; Microsoft documentation for the FillRect function
        ; for more information.
        ;-----------------------------------------------------

        invoke SetRect, ADDR rc, 0, 0, 400, 300
        invoke GetStockObject, WHITE_BRUSH
        invoke FillRect, hdcBM, ADDR rc, eax

       ;-------------------------------------
        ; Release the DC for the clent area.
        ;------------------------------------

        invoke ReleaseDC, NULL, hdcClient

      CASE WM_CTLCOLORDLG

        ;------------------------------------------
        ; Set the client area background to white.
        ;------------------------------------------

        invoke GetStockObject, WHITE_BRUSH
        ret

      CASE WM_PAINT

        ;--------------------------------------------------------------
        ; This code displays our bitmap by copying it to the paint DC.
        ;--------------------------------------------------------------

        invoke BeginPaint, hwndDlg, ADDR ps

        invoke CreateCompatibleBitmap, ps.hdc, 400, 300
        push eax
        invoke SelectObject, ps.hdc, eax
        push eax

        invoke BitBlt, ps.hdc, 0, 0, 400, 300, hdcBM, 0, 0, SRCCOPY

        pop eax
        invoke SelectObject, ps.hdc, eax
        pop eax
        invoke DeleteObject, eax

        invoke EndPaint, hwndDlg, ADDR ps

      CASE WM_SETCURSOR

        ;------------------------------------------------------------
        ; If we have a custom cursor and it is the current cursor,
        ; to prevent the system from replacing it with the class
        ; cursor each time the cursor is moved, we must process
        ; this message and return TRUE.
        ;------------------------------------------------------------

        .IF hCursor && fCursor
          mov eax, 1
          ret
        .ENDIF

      CASE WM_LBUTTONDOWN

        ;----------------------------------------
        ; Confine the cursor to the client area.
        ;----------------------------------------

        mov pt0.x, 0
        mov pt0.y, 0
        mov pt1.x, 399
        mov pt1.y, 299
        invoke ClientToScreen, hwndDlg, ADDR pt0
        invoke ClientToScreen, hwndDlg, ADDR pt1
        invoke SetRect, ADDR rc, pt0.x, pt0.y, pt1.x, pt1.y
        invoke ClipCursor, ADDR rc

        ;------------------------------------------
        ; Start a new line by updating the current
        ; position to the cursor coordinates.
        ;------------------------------------------

        invoke MoveToEx, hdcBM, loword(lParam), hiword(lParam), NULL

      CASE WM_LBUTTONUP

        ;---------------------
        ; Release the cursor.
        ;---------------------

        invoke ClipCursor, NULL

      CASE WM_MOUSELEAVE

        ;-------------------------------------------------
        ; Clear the cursor flag so on the next mouse move
        ; the system will revert to the class cursor.
        ;-------------------------------------------------

        mov fCursor, 0

      CASE WM_MOUSEMOVE

        .IF fCursor == 0

          mov fCursor, 1

          ;---------------------------------------------------------
          ; If we have a custom cursor, make it the current cursor.
          ;---------------------------------------------------------

          .IF hCursor
            invoke SetCursor, hCursor
          .ENDIF

          ;------------------------------------------
          ; Start a new line by updating the current
          ; position to the cursor coordinates.
          ;------------------------------------------

          invoke MoveToEx, hdcBM, loword(lParam), hiword(lParam), NULL

          ;-----------------------------------------------------
          ; Set up to receive a WM_MOUSELEAVE notification when
          ; the mouse pointer leaves the client area. Note that
          ; only one notification will be generated for each
          ; call to TrackMouseEvent.
          ;-----------------------------------------------------

          mov eax, hwndDlg
          mov tme.hwndTrack, eax
          invoke TrackMouseEvent, ADDR tme

        .ELSEIF wParam & MK_LBUTTON

          ;----------------------------
          ; Continue the current line.
          ;----------------------------

          invoke LineTo, hdcBM, loword(lParam), hiword(lParam)

          ;---------------------------------------------------
          ; Force a repaint of the client area so the display
          ; will be updated to match the bitmap.
          ;---------------------------------------------------

          invoke InvalidateRect, hwndDlg, NULL, FALSE

        .ENDIF

      CASE WM_COMMAND

        SWITCH loword(wParam)

          CASE IDCANCEL

            ;--------------------
            ; Clean up and exit.
            ;--------------------

            .IF hCursor
              invoke DestroyCursor, hCursor
            .ENDIF

            invoke DeleteObject, hpen
            invoke DeleteObject, hbmp
            invoke DeleteDC, hdcBM

            invoke EndDialog, hwndDlg, NULL

        ENDSW

      CASE WM_CLOSE

        ;--------------------
        ; Clean up and exit.
        ;--------------------

        .IF hCursor
          invoke DestroyCursor, hCursor
        .ENDIF

        ;------------------------------------------------
        ; The CreateBitmapFile source is in support.asm.
        ;------------------------------------------------

        invoke CreateBitmapFile, hbmp, 4, chr$("junk.bmp")

        invoke DeleteObject, hpen
        invoke DeleteObject, hbmp
        invoke DeleteDC, hdcBM

        invoke EndDialog, hwndDlg, NULL

    ENDSW

    xor eax, eax
    ret

DialogProc endp

;=========================================================================
start:
;=========================================================================

    ;---------------------------------------------------------------
    ; Get a width and height in DTUs that will size the client area
    ; close to the specified pixel width and height, regardless of
    ; the system font settings.
    ;---------------------------------------------------------------

    invoke ClientPixelWidthToDialogWidth, 400
    mov dtuX, eax
    invoke ClientPixelHeightToDialogHeight, 300
    mov dtuY, eax

    invoke DefineDialog, chr$("Mouse Draw Demo"),
                         WS_OVERLAPPED or WS_SYSMENU \
                         or DS_CENTER,
                         0, 0, dtuX, dtuY,
                         0, 0,
                         0

    invoke CreateModalDialog, 0, ADDR DialogProc, 0

    invoke ExitProcess, 0

;=========================================================================

end start
